import asyncio
import websockets
import json
import errno
import os

ist = None
prd = None
exg = None
exp = None


def get(ws, exchange, InstrumentType=None, Product=None, Expiry=None):
    if exchange == "":
        print("Exchange is mandatory.")
    else:
        exg = exchange

    ist = InstrumentType
    prd = Product
    exp = Expiry
    asyncio.get_event_loop().run_until_complete(
        mass_subscribe_n_stream(ws, exg, ist, prd, exp))
    return


async def mass_subscribe_n_stream(ws, exg, ist, prd, exp):
    try:
        print("1")
        req_msg = '{"MessageType":"GetOptionTypes","Exchange":"' + exg + '"'
        # if exg != "":
        #     req_msg = req_msg + '"Exchange":"' + exg + '"'
        if ist is not None:
            req_msg = req_msg + ',"InstrumentType":"' + ist + '"'
        if prd  is not None:
            req_msg = req_msg + ',"Product":"' + prd + '"'
        if exp  is not None:
            req_msg = req_msg + ',"Expiry":"' + exp + '"'
        print("2")
        req_msg = str(req_msg + '}')
        await ws.send(req_msg)
        print("Response : " + req_msg)
        await get_msg(ws)
        print("4")
    except:
        return "Error"


async def get_msg(ws):
    print("3")
    while True:
        try:
            message = await ws.recv()
        except websockets.ConnectionClosedOK:
            break
        print(message)



import asyncio
import websockets

url = None
key = None
exg = None
sym = None
msg = None
prc = None
prd = None
iss = None


def get(ws, exchange, symbol, periodicity, period,isShortIdentifiers):
    exg = exchange
    sym = symbol
    prc = periodicity
    prd = period
    iss = isShortIdentifiers
    asyncio.get_event_loop().run_until_complete(mass_subscribe_n_stream(ws, exg, sym, prc, prd, iss))
    return


async def mass_subscribe_n_stream(ws, exg, sym, prc, prd, iss):
    try:
        req_msg = str(
            '{"MessageType":"GetSnapshot","Exchange":"' + exg + '","Periodicity":"' + prc + '","Period":' + f'{prd}' + ',"isShortIdentifiers":"' + iss + '","InstrumentIdentifiers":' + sym + '}')
        await ws.send(req_msg)
        print("Request : " + req_msg)
        await get_msg(ws)  # Listens for the tick data until market close
    except:
        return msg


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
        except websockets.ConnectionClosedOK:
            break
        print(message)



import asyncio
import websockets

url = None
key = None
exg = None
sym = None
msg = None


def get(ws, exchange, symbol):
    exg = exchange
    sym = symbol
    asyncio.get_event_loop().run_until_complete(mass_subscribe_n_stream(ws, exg, sym))
    return


def stop(ws, exchange, symbol):
    exg = exchange
    sym = symbol
    asyncio.get_event_loop().run_until_complete(mass_unsubscribe_n_stream(ws, exg, sym))
    return


async def mass_subscribe_n_stream(ws, exg, sym):
    req_msg = str('{"MessageType":"SubscribeRealtime","Exchange":"' + exg + '","Unsubscribe":"false","InstrumentIdentifier":"' + sym + '"}')
    print("Request : " + req_msg)
    await ws.send(req_msg)
    await get_msg(ws)
    return


async def mass_unsubscribe_n_stream(ws, exg, sym):
    req_msg = str('{"MessageType":"SubscribeRealtime","Exchange":"' + exg + '","Unsubscribe":"true","InstrumentIdentifier":"' + sym + '"}')
    print("Request : " + req_msg)
    await ws.send(req_msg)
    await get_msg(ws)
    return


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
        except websockets.ConnectionClosedOK:
            break
        print(message)


import asyncio
import websockets

url = None
key = None
exg = None
sym = None
msg = None
prc = None
prd = None
frm = None
to = None
iss = None
rno = None
utg = None


def getbyperiod(ws, exchange, InstrumentIdentifier, periodicity, period, fromtime, totime, usertag, isShortIdentifiers):
    if exchange is None:
        print("Exchange is mandatory.")
    else:
        exg = exchange

    if InstrumentIdentifier is None:
        print("InstrumentIdentifier / Symbol is mandatory.")
    else:
        sym = InstrumentIdentifier

    if periodicity is None:
        prc = 'MINUTE'
    else:
        prc = periodicity

    if period is None:
        prd = '1'
    else:
        prd = period

    asyncio.get_event_loop().run_until_complete(
        mass_subscribe_n_streamft(ws, exg, sym, prc, prd, fromtime, totime, isShortIdentifiers, usertag))
    return


def getcaldle(ws, exchange, InstrumentIdentifier, periodicity, period, max, usertag, isShortIdentifiers):
    if exchange is None:
        print("Exchange is mandatory.")
    else:
        exg = exchange

    if InstrumentIdentifier is None:
        print("Symbol is mandatory.")
    else:
        sym = InstrumentIdentifier

    if periodicity is None:
        prc = 'MINUTE'
    else:
        prc = periodicity

    if period is None:
        prd = '1'
    else:
        prd = period

    if max is None:
        rno = max
    else:
        rno = 10

    if isShortIdentifiers == "":
        iss = 'false'
    else:
        iss = isShortIdentifiers

    if usertag is None:
        utg = ''
    else:
        utg = usertag

    asyncio.get_event_loop().run_until_complete(
        mass_subscribe_n_stream(ws, exg, sym, prc, prd, iss, rno, utg))
    return


async def mass_subscribe_n_streamft(ws, exg, sym, prc, prd, frm, to, iss, utg):
    print('Call From and To')
    try:
        if iss != "" and utg != "":
            req_msg = '{"MessageType":"GetHistory","Exchange":"' + exg + '","InstrumentIdentifier":"' + sym + '","Periodicity":"' + prc + '","Period":"' + str(
                prd) + '","From":' + frm + ',"To":' + to + ',"isShortIdentifier":"' + iss + '","UserTag":"' + utg + '"}'
        elif iss == "" and utg != "":
            req_msg = '{"MessageType":"GetHistory","Exchange":"' + exg + '","InstrumentIdentifier":"' + sym + '","Periodicity":"' + prc + '","Period":"' + str(
                prd) + '","From":' + frm + ',"To":' + to + ',"isShortIdentifier":"false","UserTag":"' + utg + '"}'
        elif iss == "" and utg == "":
            req_msg = '{"MessageType":"GetHistory","Exchange":"' + exg + '","InstrumentIdentifier":"' + sym + '","Periodicity":"' + prc + '","Period":"' + str(
                prd) + '","From":' + frm + ',"To":' + to + ',"isShortIdentifier":"false"}'
        elif iss != "" and utg == "":
            req_msg = '{"MessageType":"GetHistory","Exchange":"' + exg + '","InstrumentIdentifier":"' + sym + '","Periodicity":"' + prc + '","Period":"' + str(
                prd) + '","From":' + frm + ',"To":' + to + ',"isShortIdentifier":"' + iss + '"}'
        await ws.send(req_msg)
        print(req_msg)
        await get_msg(ws)
    except:
        print(msg)


async def mass_subscribe_n_stream(ws, exg, sym, prc, prd, iss, rno, utg):
    print("In MAX...")
    req_msg = None
    try:
        if iss != "" and utg != "":
            req_msg = '{"MessageType":"GetHistory","Exchange":"' + exg + '","InstrumentIdentifier":"' + sym + '","Periodicity":"' + prc + '","Period":' + str(
                prd) + ',"Max":' + str(rno) + ',"isShortIdentifier":"' + iss + '","UserTag":"' + utg + '"} '
        elif iss != "" and utg == "":
            req_msg = '{"MessageType":"GetHistory","Exchange":"' + exg + '","InstrumentIdentifier":"' + sym + '","Periodicity":"' + prc + '","Period":' + str(
                prd) + ',"Max":' + str(rno) + ',"isShortIdentifier":"' + iss + '"}'
        elif iss == "" and utg == "":
            req_msg = '{"MessageType":"GetHistory","Exchange":"' + exg + '","InstrumentIdentifier":"' + sym + '","Periodicity":"' + prc + '","Period":' + str(
                prd) + ',"Max":' + str(rno) + ',"isShortIdentifier":"false"' + '"}'
        elif iss == "" and utg != "":
            req_msg = '{"MessageType":"GetHistory","Exchange":"' + exg + '","InstrumentIdentifier":"' + sym + '","Periodicity":"' + prc + '","Period":' + str(
                prd) + ',"Max":' + str(rno) + ',"isShortIdentifier":"false"' + '","UserTag":"' + utg + '"}'
        await ws.send(str(req_msg))
        print("Request : " + req_msg)
        await get_msg(ws)
    except:
        print("Exception : " + str(msg))


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
        except websockets.ConnectionClosedOK:
            break
        print(message)



import asyncio
import websockets
from datetime import datetime
import json
import time
import errno

url = None
key = None
exg = None
sym = None
msg = None


def get(ws):
    asyncio.get_event_loop().run_until_complete(mass_subscribe_n_stream(ws))
    return


async def mass_subscribe_n_stream(ws):
    try:
        req_msg = str('{"MessageType":"GetExchanges"}')
        await ws.send(req_msg)
        print(req_msg)
        await get_msg(ws)  # Listens for the tick data until market close
    except:
        return msg


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
        except websockets.ConnectionClosedOK:
            break
        print(message)



import asyncio
import websockets
import json
import errno
import os

exg = None
ist = None


def get(ws,Exchange, InstrumentType=None):
    exg = Exchange
    ist = InstrumentType
    asyncio.get_event_loop().run_until_complete(
        mass_subscribe_n_stream(ws, exg, ist))
    return


async def mass_subscribe_n_stream(ws, exg, ist):
    try:
        req_msg = '{"MessageType":"GetProducts","Exchange":"' + exg
        if ist is not None:
            req_msg = req_msg + '",InstrumentType":"' + ist

        req_msg = str(req_msg + '"}')
        await ws.send(req_msg)
        print("Request : " + req_msg)
        await get_msg(ws)
    except:
        return "Error"


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
        except websockets.ConnectionClosedOK:
            break
        print(message)



import asyncio
import websockets
import json
import errno
import os

url = None
key = None
exg = None


def get(ws, exchange, instrumenttype=None, product=None):
    # if endpoint == "":
    #     print("Endpoint is mandatory.")
    # else:
    #     url = endpoint
    #
    # if apikey == "":
    #     print("APIkey is mandatory.")
    # else:
    #     key = apikey

    if exchange == "":
        print("Exchange is mandatory.")
    else:
        exg = exchange

    asyncio.get_event_loop().run_until_complete(
        mass_subscribe_n_stream(ws, exg,instrumenttype, product))
    return


async def mass_subscribe_n_stream(ws, exg, ist, prd):
    try:
        req_msg = '{"MessageType":"GetExpiryDates","Exchange":"' + exg + '"'
        if ist is not None:
            req_msg = req_msg + '"InstrumentType":"' + ist + '"'
        if prd is not None:
            req_msg = req_msg + '"Product":"' + prd + '"'
        req_msg = str(req_msg + '}')
        print("Request : " + req_msg)
        await ws.send(req_msg)
        await get_msg(ws)
    except:
        print('In Exception...' + os.error)
        return


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
        except websockets.ConnectionClosedOK:
            break
        print(message)



import asyncio
import websockets


exg = None
sym = None
msg = None
isi = None


def get(ws, exchange, symbols,isShortIdentifiers):
    exg = exchange
    sym = symbols
    isi = isShortIdentifiers
    asyncio.get_event_loop().run_until_complete(mass_subscribe_n_stream(ws, exg, sym, isi))
    return


async def mass_subscribe_n_stream(ws, exg, sym, isi):
    try:
        req_msg = str('{"MessageType":"GetLastQuoteArrayShort","Exchange":"' + exg + '","isShortIdentifiers":"' + isi + '","InstrumentIdentifiers":' + str(sym) + '}')
        await ws.send(req_msg)
        print("Request : " + req_msg)
        await get_msg(ws)
    except:
        return msg


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
        except websockets.ConnectionClosedOK:
            break
        print(message)


import asyncio
import websockets
from datetime import datetime
import json
import time
import errno

url = None
key = None
src = None
exg = None
ity = None
prd = None
exp = None
otp = None
srp = None
oa = None


def get(ws, exchange, search, instrumenttype=None, product=None, expiry=None, optiontype=None, strikeprice=None,
        onlyactive=None):
    print("In Get...")
    # if endpoint == "":
    #     print("Endpoint is mandatory.")
    # else:
    #     url = endpoint
    #
    # if apikey == "":
    #     print("APIkey is mandatory.")
    # else:
    #    key = apikey

    if exchange == "":
        print("Exchange is mandatory.")
    else:
        exg = exchange

    if search == "":
        print("Search is mandatory.")
    else:
        src = search

    if instrumenttype == "":
        ity = ''
    else:
        ity = instrumenttype

    if product == "":
        prd = ''
    else:
        prd = product

    if expiry == "":
        exp = ''
    else:
        exp = expiry

    if optiontype == "":
        otp = ''
    else:
        otp = optiontype

    if strikeprice == "":
        srp = ''
    else:
        srp = strikeprice

    if onlyactive == "":
        oa = 'true'
    else:
        oa = onlyactive

    asyncio.get_event_loop().run_until_complete(
        mass_subscribe_n_stream(ws, exg, src, ity, prd, exp, otp, srp, oa))
    return


async def mass_subscribe_n_stream(ws, exg, src, ity, prd, exp, otp, srp, oa):
    try:
        req_msg = '{"MessageType":"GetInstrumentsOnSearch",'
        if exg != None:
            req_msg = str(req_msg) + '"Exchange":"' + exg + '"'
        if src != None:
            req_msg = str(req_msg) + ',"Search":"' + src + '"'
        if ity != None:
            req_msg = str(req_msg) + ',"InstrumentType":"' + ity + '"'
        if prd != None:
            req_msg = str(req_msg) + ',"Product":"' + prd + '"'
        if exp != None:
            req_msg = str(req_msg) + ',"Expiry":"' + exp + '"'
        if otp != None:
            req_msg = str(req_msg) + ',"optionType":"' + otp + '"'
        if srp != None:
            req_msg = str(req_msg) + ',"strikePrice":"' + srp + '"'
        if oa != None:
            req_msg = str(req_msg) + ',"onlyActive":"' + oa + '"'
        req_msg = str(req_msg + '}')
        print("Request : " + req_msg)
        await ws.send(req_msg)
        await get_msg(ws)
    except:
        print('In Exception...' + errno)
        return


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
        except websockets.ConnectionClosedOK:
            break
        print(message)



import asyncio
import websockets
import json
import errno

url = None
key = None
exg = None
sym = None
msg = None
prc = None
prd = None
frm = None
to = None
iss = None
rno = None
utg = None


def get(endpoint, apikey, exchange, symbol, periodicity, period, fromtime, totime, isShortIdentifiers, recno, usertag):
    print("In Get...")
    if endpoint is None:
        print("Endpoint is mandatory.")
    else:
        url = endpoint

    if apikey is None:
        print("APIkey is mandatory.")
    else:
        key = apikey

    if exchange is None:
        print("Exchange is mandatory.")
    else:
        exg = exchange

    if symbol is None:
        print("Symbol is mandatory.")
    else:
        sym = symbol

    if periodicity is None:
        prc = 'MINUTE'
    else:
        prc = periodicity

    if period is None:
        prd = '1'
    else:
        prd = period

    frm = fromtime
    to = totime

    rno = recno

    if isShortIdentifiers == "":
        iss = 'false'
    else:
        iss = isShortIdentifiers

    if usertag is None:
        utg = ' '
    else:
        utg = usertag

    print("iss : " + iss)

    asyncio.get_event_loop().run_until_complete(
        mass_subscribe_n_stream(url, key, exg, sym, prc, prd, frm, to, iss, rno, utg))
    return


async def mass_subscribe_n_stream(url, key, exg, sym, prc, prd, frm, to, iss, rno, utg):
    try:
        print("1 Here...")
        ws = await asyncio.wait_for(websockets.connect(url), timeout=60)

        await authenticate(ws, key, exg, sym, prc, prd, frm, to, iss, rno, utg)
        await get_msg(ws)
        print("2 Here...")
    except:
        return msg


async def authenticate(ws, key, exg, sym, prc, prd, frm, to, iss, rno, utg):
    try:
        msg = 'Performing Authentication'
        print(msg)
        authentication_msg = json.dumps({
            "MessageType": "Authenticate",
            "Password": key
        })
        authenticated = False
        await ws.send(authentication_msg)
        while not authenticated:
            response = await ws.recv()
            response = json.loads(response)
            print(response)
            if response['MessageType'] == "AuthenticateResult":
                if response['Complete']:
                    await subscribe_n_stream(ws, key, exg, sym, prc, prd, frm, to, iss, rno, utg)
                    await get_msg(ws)
                else:
                    await ws.send(authentication_msg)
        msg = authenticated
    except:
        return msg


async def subscribe_n_stream(ws, key, exg, sym, prc, prd, frm, to, iss, rno, utg):
    try:
        if frm != "" and to != "" and to != "" and rno != "" and utg != "":
            req_msg = str(
                '{"MessageType":"GetHistoryAfterMarket","Exchange":"' + exg + '","InstrumentIdentifier":"' + sym
                + '","Periodicity":"' + prc + '","Period":"' + str(prd) + '","From":"' + frm + '","To":"' + to
                + '", "Max":"' + str(rno) + '","UserTag":"' + utg + '","isShortIdentifier":"' + iss + '"}')
        elif frm != "" and to != "" and to != "" and rno != "" and utg == "":
            req_msg = str(
                '{"MessageType":"GetHistoryAfterMarket","Exchange":"' + exg + '","InstrumentIdentifier":"' + sym
                + '","Periodicity":"' + prc + '","Period":"' + str(prd) + '","From":"' + frm + '","To":"' + to
                + '", "Max":"' + str(rno) + '","isShortIdentifier":"' + iss + '"}')
            await ws.send(req_msg)
        elif frm != "" and to != "" and to != "" and rno == "" and utg == "":
            req_msg = str(
                '{"MessageType":"GetHistoryAfterMarket","Exchange":"' + exg + '","InstrumentIdentifier":"' + sym
                + '","Periodicity":"' + prc + '","Period":"' + str(prd) + '","From":' + frm + ',"To":' + to
                + ',"isShortIdentifier":' + iss + '}')
            await ws.send(req_msg)
            print(req_msg)
        elif frm != "" and to == "" and rno == "" and utg == "":
            req_msg = str(
                '{"MessageType":"GetHistoryAfterMarket","Exchange":"' + exg + '","InstrumentIdentifier":"' + sym
                + '","Periodicity":"' + prc + '","Period":' + str(prd) + ',"From":' + frm + '}')
            await ws.send(req_msg)
            print(req_msg)
    except:
        print(msg)
        return


async def get_msg(ws):
    try:
        response = await ws.recv()
        response = json.loads(response)
        if response['MessageType'] != "RealtimeResult":
            pass
        else:
            print("XXX : " + response)
    except:
        print('In Exception...' + errno)



from gfdlws import lastquote
from gfdlws import lastquoteshort
from gfdlws import lastquoteshortclose
from gfdlws import lastquotearray
from gfdlws import lastquotearrayshort
from gfdlws import lastquotearrayshortclose
from gfdlws import realtime
from gfdlws import snapshot
from gfdlws import history
from gfdlws import realsnapshot
from gfdlws import exchanges
from gfdlws import ws
from gfdlws import instrumentsonsearch
from gfdlws import instruments
from gfdlws import instrumenttypes
from gfdlws import products
from gfdlws import expirydates
from gfdlws import optiontypes
from gfdlws import strikeprices
from gfdlws import serverinfo
from gfdlws import limitation
from gfdlws import marketmessages
from gfdlws import exchangemessages
from gfdlws import lastquoteoptionchain
from gfdlws import exchangesnapshot
from gfdlws import historyaftermarket
from gfdlws import realtimegreeks
from gfdlws import lastquoteoptiongreeks
from gfdlws import lastquotearrayoptiongreeks
from gfdlws import lastquoteoptiongreekschain


import asyncio
import websockets
import json

url = None
key = None
exg = None
sym = None
msg = None
ws = None


def connect(endpoint, apikey):
    ws = asyncio.get_event_loop().run_until_complete(mass_subscribe_n_stream(endpoint, apikey))
    return ws


async def mass_subscribe_n_stream(endpoint, apikey):
    ws = await asyncio.wait_for(websockets.connect(endpoint, max_size=2 ** 50), timeout=60)
    stst = await authenticate(ws, apikey)  # Authenticates the connection
    if stst == 'Key Authenticated!!!':
        return ws
    else:
        print('Not connected. Trying again. Please wait...')
        stst = await authenticate(ws, apikey)


async def authenticate(ws, key):
    try:
        msg = 'Performing Authentication'
        print(msg)
        authentication_msg = json.dumps({
            "MessageType": "Authenticate",
            "Password": key
        })
        authenticated = False
        await ws.send(authentication_msg)
        while not authenticated:  # Stay in this loop until successful authentication.
            response = await ws.recv()
            response = json.loads(response)
            print(response)
            if response['MessageType'] == "AuthenticateResult":
                if response['Complete']:
                    status = 'Key Authenticated!!!'
                    return status
                else:
                    print('Not connected. Trying again. Please wait...')
                    await ws.send(authentication_msg)
    except:
        return msg


import asyncio
import websockets

exg = None
tkn = None
msg = None
ws = None


def get(ws, exchange, token):
    exg = exchange
    tkn = token
    # isi = isShortIdentifier
    asyncio.get_event_loop().run_until_complete(mass_subscribe_n_stream(ws, exg, tkn))
    return


async def mass_subscribe_n_stream(ws, exg, tkn):
    try:
        req_msg = str(
            '{"MessageType":"GetLastQuoteOptionGreeks","Exchange":"' + exg + '","Token":"' + tkn + '"}')
        await ws.send(req_msg)
        print("Request : " + req_msg)
        await get_msg(ws)
    except:
        return msg


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
        except websockets.ConnectionClosedOK:
            break
        print(message)



import asyncio
import websockets
import json
import errno
import os

exg = None


def get(ws, exchange, instrumenttype=None, product=None, expiry=None, optiontype=None):
    if exchange == "":
        print("Exchange is mandatory.")
    else:
        exg = exchange
    asyncio.get_event_loop().run_until_complete(
        mass_subscribe_n_stream(ws, exg, instrumenttype, product, expiry, optiontype))
    return


async def mass_subscribe_n_stream(ws, exg, ist, prd, exp, opt):
    try:
        req_msg = '{"MessageType":"GetStrikePrices", "Exchange":"' + exg + '"'
        # if ist is not None:
        #     req_msg = req_msg + '"Exchange":"' + exg + '"'
        if ist is not None:
            req_msg = req_msg + ',"InstrumentType":"' + ist + '"'
        if prd is not None:
            req_msg = req_msg + ',"Product":"' + prd + '"'
        if exp is not None:
            req_msg = req_msg + ',"Expiry":"' + exp + '"'

        req_msg = str(req_msg + '}')
        await ws.send(req_msg)
        print("Response : " + req_msg)
        await get_msg(ws)
    except:
        return "Error"


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
        except websockets.ConnectionClosedOK:
            break
        print(message)



import asyncio
import websockets
import json
import errno
import os


def get(ws):
    asyncio.get_event_loop().run_until_complete(
        mass_subscribe_n_stream(ws))
    return


async def mass_subscribe_n_stream(ws):
    try:
        req_msg = str('{"MessageType":"GetServerInfo"}')
        await ws.send(req_msg)
        print("Request : " + req_msg)
        await get_msg(ws)
    except:
        return "Error"


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
        except websockets.ConnectionClosedOK:
            break
        print(message)
